@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
  * Copyright © 2017 Igalia S.L.
+ * Copyright © 2018 Jan-Michael Brummer
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@ struct _EphySuggestion {
   DzlSuggestion parent;
 
   char *unescaped_title;
+  cairo_surface_t *favicon;
 };
 
 G_DEFINE_TYPE (EphySuggestion, ephy_suggestion, DZL_TYPE_SUGGESTION)
@@ -87,6 +89,16 @@ ephy_suggestion_replace_typed_text (DzlSuggestion *self,
   return g_strdup (url);
 }
 
+cairo_surface_t *
+ephy_suggestion_get_icon_surface (DzlSuggestion *self,
+                                  GtkWidget     *widget)
+{
+  EphySuggestion *suggestion = EPHY_SUGGESTION (self);
+
+  return suggestion->favicon;
+}
+
+
 static void
 ephy_suggestion_class_init (EphySuggestionClass *klass)
 {
@@ -97,6 +109,7 @@ ephy_suggestion_class_init (EphySuggestionClass *klass)
   object_class->set_property = ephy_suggestion_set_property;
 
   dzl_suggestion_class->replace_typed_text = ephy_suggestion_replace_typed_text;
+  dzl_suggestion_class->get_icon_surface = ephy_suggestion_get_icon_surface;
 
   obj_properties[PROP_UNESCAPED_TITLE] =
     g_param_spec_string ("unescaped-title",
@@ -172,3 +185,11 @@ ephy_suggestion_get_uri (EphySuggestion *self)
 
   return dzl_suggestion_get_id (DZL_SUGGESTION (self));
 }
+
+void
+ephy_suggestion_set_favicon (EphySuggestion  *self,
+                             cairo_surface_t *favicon)
+{
+  self->favicon = favicon;
+  g_object_notify (G_OBJECT (self), "icon");
+}
