@@ -471,7 +471,7 @@ ephy_web_application_free (EphyWebApplication *app)
   g_free (app->icon_url);
   g_free (app->url);
   g_free (app->desktop_file);
-  g_slice_free (EphyWebApplication, app);
+  g_free (app);
 }
 
 
@@ -494,7 +494,7 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   if (!id)
     return NULL;
 
-  app = g_slice_new0 (EphyWebApplication);
+  app = g_new0 (EphyWebApplication, 1);
   app->id = g_strdup (id);
 
   app->desktop_file = get_app_desktop_filename (id);
@@ -726,7 +726,7 @@ ephy_web_application_save (EphyWebApplication *app)
     g_free (name);
 
     icon = g_key_file_get_string (key, "Desktop Entry", "Icon", NULL);
-    if (g_strcmp0 (icon, app->icon_url) != 0) {
+    if (g_strcmp0 (name, app->icon_url) != 0) {
       changed = TRUE;
       g_key_file_set_string (key, "Desktop Entry", "Icon", app->icon_url);
     }
