@@ -521,8 +521,7 @@ entry_key_press_cb (GtkEntry          *entry,
 
 
   if (event->keyval == GDK_KEY_Escape && state == 0) {
-    ephy_location_entry_reset_internal (location_entry, TRUE);
-    /* don't return TRUE since we want to cancel the autocompletion popup too */
+    ephy_location_entry_reset_internal (location_entry, FALSE);
   }
 
   if (event->keyval == GDK_KEY_l && state == GDK_CONTROL_MASK) {
@@ -545,7 +544,9 @@ entry_clear_activate_cb (GtkMenuItem       *item,
                          EphyLocationEntry *entry)
 {
   entry->block_update = TRUE;
+  g_signal_handlers_block_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
   gtk_entry_set_text (GTK_ENTRY (entry->url_entry), "");
+  g_signal_handlers_unblock_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
   entry->block_update = FALSE;
   entry->user_changed = TRUE;
 }
@@ -556,8 +557,10 @@ paste_received (GtkClipboard      *clipboard,
                 EphyLocationEntry *entry)
 {
   if (text) {
+    g_signal_handlers_block_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
     gtk_entry_set_text (GTK_ENTRY (entry->url_entry), text);
     ephy_location_entry_activate (entry);
+    g_signal_handlers_unblock_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
   }
 }
 
@@ -728,6 +731,8 @@ button_box_size_allocated_cb (GtkWidget    *widget,
    *    including border spacing 5).
    *  - progress margin-right is used to allow progress bar below icons on the
    *    right side.
+   *
+   * FIXME: Loading CSS during size_allocate is ILLEGAL and BROKEN.
    */
   css = g_strdup_printf (".url_entry { padding-right: %dpx; }"\
                          ".url_entry progress { margin-right: -%dpx; }",
@@ -866,7 +871,7 @@ free_prefetch_helper (PrefetchHelper *helper)
 {
   soup_uri_free (helper->uri);
   g_object_unref (helper->entry);
-  g_slice_free (PrefetchHelper, helper);
+  g_free (helper);
 }
 
 static gboolean
@@ -897,7 +902,7 @@ schedule_dns_prefetch (EphyLocationEntry *entry, guint interval, const gchar *ur
   if (entry->dns_prefetch_handler)
     g_source_remove (entry->dns_prefetch_handler);
 
-  helper = g_slice_new0 (PrefetchHelper);
+  helper = g_new0 (PrefetchHelper, 1);
   helper->entry = g_object_ref (entry);
   helper->uri = uri;
 
@@ -982,7 +987,9 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 void
 ephy_location_entry_undo_reset (EphyLocationEntry *entry)
 {
+  g_signal_handlers_block_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
   gtk_entry_set_text (GTK_ENTRY (entry->url_entry), entry->saved_text);
+  g_signal_handlers_unblock_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
   entry->can_redo = FALSE;
   entry->user_changed = TRUE;
 }
@@ -1203,13 +1210,20 @@ ephy_location_entry_set_progress (EphyLocationEntry *entry,
                                   gdouble            fraction,
                                   gboolean           loading)
 {
+  gdouble current_progress;
+
   if (entry->progress_timeout) {
     g_source_remove (entry->progress_timeout);
     entry->progress_timeout = 0;
  }
 
   if (!loading) {
-    gtk_entry_set_progress_fraction (GTK_ENTRY (entry->url_entry), 0);
+    /* Setting progress to 0 when it is already 0 can actually cause the
+     * progress bar to be shown. Yikes....
+     */
+    current_progress = gtk_entry_get_progress_fraction (GTK_ENTRY (entry->url_entry));
+    if (current_progress != 0.0)
+      gtk_entry_set_progress_fraction (GTK_ENTRY (entry->url_entry), 0.0);
     return;
   }
 
